/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LightSMS.
 *
 * The Initial Developer of the Original Code is
 * Byungwook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

const nsILightSMS = Components.interfaces.nsILightSMS;

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{20fd7fe8-248d-11dc-8314-0800200c9a66}");

// description
const CLASS_NAME = "LightSMS Service";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/LightSMS;1";

const DATA_VASP_ID=0;
const DATA_VAS_ID=1;
const DATA_SMS_IP=2;
const DATA_SMS_PORT=3;
const DATA_SMS_FREERCVFILE=4;
const DATA_USER_KEY=5;
const DATA_USER_LEVEL=6;
const DATA_USER_MDN=7;
const DATA_USER_NUM=8;
const DATA_FREE_COUNT=9;
const DATA_CASH_KEY=10;
const DATA_COUPON_COUNT=11;

const STAGE_READY=20;
const STAGE_SEND=21;

const CMD_LOGIN=0;
const CMD_SEND=1;

const dout=Components.utils.reportError;

/***********************************************************
class definition
***********************************************************/

//class constructor
function LightSMS() {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  this.branch = prefService.getBranch("lightsms.");
  this.branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  this.initHandler();
}

// class definition
LightSMS.prototype = {
  userData:[],
  listeners : new Array(),
  stage:0,
  onreq:false,
  cmd:CMD_LOGIN,
  recv:null,
  msg:null,
  retry:0,
  initHandler: function(){
    if(this.timer)this.timer.cancel();
    if(this.handler)this.handler.stop();
    var s=this.getPref("username");
    s=s.split("@");
    if(s[1]=="tworld.co.kr"){
      this.handler=new SMS_Tworld();
    }else{
      this.handler=new SMS_NateOn();
    }
    this.handler.main=this;
    this.handler.reader=new HTMLReader("euc-kr");
    this.userData=[];
    this.userData[DATA_COUPON_COUNT]=0;
    this.retry=0;
    this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_TRY);
    this.handler.doLogin(CMD_LOGIN);
  },
  sendSMS: function(recv,msg){
    this.recv=recv;
    this.msg=msg;
    if(this.timer)this.timer.cancel();
    this.handler.stop();
    if(!this.userData[DATA_USER_KEY]){
      this.retry=0;
      this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_TRY);
      this.handler.doLogin(CMD_SEND);
    }else this.handler.send();
  },
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsILightSMS) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
LightSMS.prototype.getPref=function(name){
  return this.branch.getCharPref(name);
}
LightSMS.prototype.setPref=function(name,value){
  this.branch.setCharPref(name,value);
}
LightSMS.prototype.addListener = function(aListener) {
  this.listeners.push(aListener);
  if(this.onreq)return;
  if(!this.userData[DATA_USER_KEY]){//not logged in
    this.retry=0;
    this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_TRY);
    this.handler.doLogin(CMD_LOGIN);
  }else{
    aListener.onStateChange(nsILightSMS.ST_LOGIN,nsILightSMS.RS_SUCC);
    aListener.onStateChange(nsILightSMS.ST_FREECOUNT,this.userData[DATA_FREE_COUNT]+this.userData[DATA_COUPON_COUNT]);
  }
}
LightSMS.prototype.removeListener = function(aListener){
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]==aListener){
        this.listeners.splice(i,1);
        break;
    }
  }
}
LightSMS.prototype.setState = function(aID,aData) {
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]){
      this.listeners[i].onStateChange(aID,aData);
    }
  }
}
LightSMS.prototype.showError = function(aData, aStatus,aHttp) {
  this.onreq=false;
  this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_FAIL);
dout("Err "+this.stage+" "+this.retry+" "+aStatus+"|"+aData);
  if (this.timer) {
    this.timer.cancel();
  } else {
    this.timer = Components.classes["@mozilla.org/timer;1"]
                  .createInstance(Components.interfaces.nsITimer);
  }
  try{
    this.timer.initWithCallback(this,
                                3000,
                                Components.interfaces.TYPE_ONE_SHOT);
  } catch (e) {
  }
}
LightSMS.prototype.notify = function(aTimer) {
//dout("reload "+this.retry);
  if(this.onreq||this.retry>3)return;
  if(!this.userData[DATA_USER_KEY]){
    ++this.retry;
    this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_RETRY);
    this.handler.doLogin(this.cmd);
  }
}

/***********************************************************
password manager
***********************************************************/
LightSMS.prototype.getPassword=function(username){
  var srvc = Components.classes["@mozilla.org/autocomplete/search;1?name=password"]
                    .getService (Components.interfaces.acIPassword);
  return srvc.getPassword("chrome://lightsms/username",username);
}
LightSMS.prototype.getNumber=function(str){
  return str.replace(/-/g,"").match(/\d{3,}/);
}
LightSMS.prototype.getDateString=function(){
      var dt=new Date();
      function getString(n){
        if(n<10)return "0"+n;
        else return n;
      }
      return dt.getFullYear()+"-"+getString(dt.getMonth()+1)+"-"+getString(dt.getDate())
              +" "+getString(dt.getHours())+":"+getString(dt.getMinutes())+":"+getString(dt.getSeconds());

}
LightSMS.prototype.onSendSucc=function(){
  this.setState(nsILightSMS.ST_SEND,nsILightSMS.RS_SUCC);
  if(this.userData[DATA_FREE_COUNT]>0)--this.userData[DATA_FREE_COUNT];
  else if(this.userData[DATA_COUPON_COUNT]>0)--this.userData[DATA_COUPON_COUNT];
  if(this.branch.getBoolPref("saveHist")){
    var str=this.msg+"\t"+this.recv+"\t"+this.getPref("sender")+"\t"+this.getDateString();
    var db = Components.classes["@mozilla.org/autocomplete/search;1?name=text-file"]
                    .getService (Components.interfaces.nsITextFile);
    db.addEntry("smshist.txt", str,true);
  }
}


/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var LightSMSFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new LightSMS()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var LightSMSModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return LightSMSFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return LightSMSModule; }


/***********************************************************
HTMLReader
***********************************************************/
function HTMLReader(aCharSet) {
  if(aCharSet)this.charSet=aCharSet;
}
HTMLReader.prototype.getHtml = function(aCallback,aURL,aPostData) {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = ioService.newURI(aURL, "ISO-8859-1", null);
  var channel = ioService.newChannelFromURI(uri);
  var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  if (aPostData) {
    var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
      .createInstance(Components.interfaces.nsIStringInputStream);
    uploadStream.setData(aPostData, aPostData.length);
    var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);
    httpChannel.requestMethod = "POST";
  }
  this.callback=aCallback;
  this.httpChannel=httpChannel;
  channel.asyncOpen(this,httpChannel);
}
HTMLReader.prototype.onStartRequest = function (aRequest, aContext) {
  this.data = "";
}
HTMLReader.prototype.onDataAvailable = function (aRequest, aContext, aStream, aSourceOffset, aLength){
  var is = Components.classes["@mozilla.org/scriptableinputstream;1"]
            .createInstance(Components.interfaces.nsIScriptableInputStream);
  is.init(aStream);
  this.data += is.read(aLength);
}

HTMLReader.prototype.onStopRequest = function (aRequest, aContext, aStatus) {
  this.httpChannel=null;
  var response=-1;
  try{
    response=aContext.QueryInterface(Components.interfaces.nsIHttpChannel).responseStatus
  }catch(e){}

  var charSet=null;
  var httpChannel=aContext.QueryInterface(Components.interfaces.nsIHttpChannel);
  if(this.charSet)charSet=this.charSet;
  else{
    var fnd=this.data.match(/<meta.+?\".+?charset=(\S+?)\s*?\"\s*?\x2f*>/);
    if(fnd)charSet=fnd[1];
    else{
      try{
        if(httpChannel.contentCharset)charSet=httpChannel.contentCharset;
      }catch(e){}
    }
  }
  if(charSet){
    var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
    									.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    uniConv.charset = charSet;
    try{
      this.data=uniConv.ConvertToUnicode(this.data);
    }catch(e){}
  }
  this.callback.callback(this.data,response,aContext);
}
HTMLReader.prototype.stop = function () {
  if(this.httpChannel){
    this.httpChannel.cancel(Components.results.NS_BINDING_ABORTED);
    this.httpChannel=null;
  }
}

function SocketReader(callback,host,port) {
  this.callback=callback;
  var transportService =
    Components.classes["@mozilla.org/network/socket-transport-service;1"]
      .getService(Components.interfaces.nsISocketTransportService);
  this.transport = transportService.createTransport(null,0,host,port,null);
  this.outstream = this.transport.openOutputStream(0,0,0);
  var stream = this.transport.openInputStream(0,0,0);
  this.instream = Components.classes["@mozilla.org/scriptableinputstream;1"]
    .createInstance(Components.interfaces.nsIScriptableInputStream);
  this.instream.init(stream);
  var pump = Components.
    classes["@mozilla.org/network/input-stream-pump;1"].
      createInstance(Components.interfaces.nsIInputStreamPump);
  pump.init(stream, -1, -1, 0, 0, false);
  pump.asyncRead(this,null);
}
SocketReader.prototype.write = function (data) {
  this.outstream.write(data,data.length);
}
SocketReader.prototype.onStartRequest = function (aRequest, aContext) {
}
SocketReader.prototype.onDataAvailable = function (aRequest, aContext, aStream, aSourceOffset, aLength){
  this.callback.callback(this.instream.read(aLength),1);
}

SocketReader.prototype.onStopRequest = function (aRequest, aContext, aStatus) {
  this.close();
  //this.callback.callback("",0);
}
SocketReader.prototype.close = function () {
  this.instream.close();
  this.outstream.close();
}

function SMS_Tworld(){
}
SMS_Tworld.prototype.doLogin = function(aCmd) {
  this.main.cmd=aCmd;
  this.main.onreq=true;
  this.main.stage=0;
  //this.reader.getHtml(this,"http://membership.tworld.co.kr/jsp/common/Logout.jsp");
  //this.reader.getHtml(this,"http://www.tworld.co.kr:8001/common/loginout/view/cm6_logout_page.jsp");
  this.reader.getHtml(this,"http://icasams.sktelecom.com:2040/icas/fc/LogOffSV?URL=http://www.tworld.co.kr:8001/common/loginout/view/cm6_logout_page.jsp");
}
SMS_Tworld.prototype.stop = function() {
  this.reader.stop();
}
SMS_Tworld.prototype.callback=function(aData, aStatus,aHttp){
  var main=this.main;
  var fnd;
  switch(main.stage){
  case 0:
    this.reader.getHtml(this,"http://www.tworld.co.kr:8001/common/loginout/view/cm6_login_page.jsp");
    break;
  case 1://get server ip
    fnd=aData.match(/SERVERIP\s+value=(\S+)>/);
    if(fnd){
      var serverIP=fnd[1];
      //out(serverIP);
      var username=main.getPref("username");
      if(!username){
        main.onreq=false;
        return;
      }
      var ar=username.split("@");
      //if(ar[1]=="tworld.co.kr")username=ar[0];
      var password=main.getPassword(username);
      if(!password){
        main.onreq=false;
        return;
      }
      this.reader.getHtml(this,"http://icasams.sktelecom.com:2040/icas/fc/LogOnSV",
          "URL="+encodeURIComponent("http://www.tworld.co.kr:8001/common/loginout/control/LoginServlet.jsp?returnURL=&kind=&popup=&cmd=")
          +"&SERVERIP="+serverIP
          +"&ID="+encodeURIComponent(ar[0])
          +"&PASSWORD="+encodeURIComponent(password)
      );
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 2:
    if(aStatus==302){
        this.reader.charSet="utf-8";
        this.reader.getHtml(this,"http://message.tworld.co.kr/mpomr/13001/main.xcg?opcode=main&vasid=13001");
        break;
    }
    main.showError(aData,aStatus,aHttp);
    break;
  case 3://find url of iframe
    fnd=aData.match(/"mainpage"[\s\S]+?src="(\S+)"/);
    if(fnd){
      this.reader.getHtml(this,fnd[1]);
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 4://find this.location.href
    fnd=aData.match(/href="(\S+)"/);
    if(fnd){
      this.reader.getHtml(this,"http://mpr.nate.com"+fnd[1]);
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 5://get values
    var reg=[ /"user_mdn"[\S\s]*?=\s*?"(\S+?)"/,
              /"user_num"[\S\s]*?=\s*?"(\S+?)"/,
              /"free_count"[\S\s]*?=\s*?"(\S+?)"/,
              /"cash_key"[\S\s]*?=\s*?"(\S+?)"/
        ];
    for(var i=0;i<reg.length;i++){
      fnd=aData.match(reg[i]);
      if(fnd){
        main.userData[i+DATA_USER_MDN]=fnd[1];
      }else{
        main.showError(aData,aStatus,aHttp);
        return;
      }
    }
    main.userData[DATA_FREE_COUNT]=parseInt(main.userData[DATA_FREE_COUNT]);
    if(!main.getPref("sender"))main.setPref("sender",main.userData[DATA_USER_MDN]);
    fnd=aData.match(/"leftFrame"[\s\S]+?src="(\S+)"/);
    if(fnd){
      this.reader.charSet=null;
      this.reader.getHtml(this,"http://mpr.nate.com"+fnd[1]);
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 6:
    var reg=[/RXA_VASP_ID\s*?=\s*?"(\S+?)"/,
              /RXA_VAS_ID\s*?=\s*?"(\S+?)"/,
              /RXA_SMS_IP\s*?=\s*?"(\S+?)"/,
              /RXA_SMS_PORT\s*?=\s*?"(\S+?)"/,
              /RXA_SMS_FREERCVFILE\s*?=\s*?"(\S+?)"/,
              /RXA_USER_KEY\s*?=\s*?"(\S+?)"/,
              /RXA_USER_LEVEL\s*?=\s*?"(\S+?)"/
        ];
    for(var i=0;i<reg.length;i++){
      fnd=aData.match(reg[i]);
      if(fnd){
        main.userData[i]=fnd[1];
      }else{
        main.showError(aData,aStatus,aHttp);
        return;
      }
    }
    main.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_SUCC);
    main.setState(nsILightSMS.ST_FREECOUNT,main.userData[DATA_FREE_COUNT]);
    main.stage=STAGE_READY;
    if(main.cmd==CMD_SEND){
      this.handler.send();
    }else main.onreq=false;
    break;
  case STAGE_SEND:
    fnd=aData.match(/<RESULT>(-*\d+)<\/RESULT>/);
    if(fnd&&fnd[1]==0){
      main.onSendSucc();
    }else{
      main.setState(nsILightSMS.ST_SEND,nsILightSMS.RS_FAIL);
    }
    main.sock.close();
    main.onreq=false;
    break;
  }
  if(main.stage<STAGE_READY)main.stage++;
}
SMS_Tworld.prototype.send=function(){
  var main=this.main;
  main.stage=STAGE_SEND;
  var sender=main.getPref("sender");
  var receiver=main.getNumber(main.recv);
  var msg=Components.classes["@mozilla.org/intl/texttosuburi;1"]
            .getService(Components.interfaces.nsITextToSubURI)
              .ConvertAndEscape("euc-kr", main.msg);

  var post="TYPE=01&VASP_ID="+main.userData[DATA_VASP_ID]
            +"&VAS_ID="+main.userData[DATA_VAS_ID]
            +"&VAS_CASH_YN=N&USER_LEVEL="+main.userData[DATA_USER_LEVEL]
            +"&USER_ID="+main.userData[DATA_USER_KEY]
            +"&USER_NUM="+main.userData[DATA_USER_NUM]
            +"&CASH_KEY="+main.userData[DATA_CASH_KEY]
            +"&FREE_COUNT="+main.userData[DATA_FREE_COUNT]
            +"&SEND_MDN="+sender+"&DELIVERY_TIME=&RECV_MDN="+receiver
            +"&CALL_MDN="+sender+"&TEXT="+msg;
  main.sock=new SocketReader(this,main.userData[DATA_SMS_IP],main.userData[DATA_SMS_PORT]);
  main.sock.write("POST /mpomr/sms_send_free.fcgi HTTP/1.0\r\n"
    +"Content-Length: "+post.length+"\r\n\r\n"
    +post);
}

function SMS_NateOn(){
}
SMS_NateOn.prototype.doLogin = function(aCmd) {
  this.main.cmd=aCmd;
  this.main.onreq=true;
  this.main.stage=0;
  this.sock=new SocketReader(this,"dpl.nate.com",5004);
  this.sock.write("PVER 1 3.617 3.0\r\n");
}
SMS_NateOn.prototype.stop = function() {
  this.reader.stop();
  if(this.sock)this.sock.close();
}
SMS_NateOn.prototype.callback=function(aData, aStatus,aHttp){
  var main=this.main;
  var username=main.getPref("username");
  switch(main.stage){
  case 0:
    if(aData.indexOf("PVER 1")==0){
      this.sock.write("AUTH 2 DES\r\n");
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 1:
    if(aData.indexOf("AUTH 2")==0){
      this.sock.write("REQS 3 DES "+username+"\r\n");
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 2:
    if(aData.indexOf("REQS 3")==0){
      var s=aData.split(" ");
      this.sock.close();
      this.sock=new SocketReader(this,s[3],s[4]);
      var t=username.split("@");
      var password=main.getPassword(username);
      if(t[1]=="nate.com")t=password+t[0];
      else t=password+username;
      this.sock.write("LSIN 1 "+username+" "+this._MD5(t)+" MD5 3.617 UTF8\r\n");
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 3:
    if(aData.indexOf("LSIN 1")==0){
      this.sock.close();
      var t=aData.split(" ");
      main.userData[DATA_USER_MDN]=t[5];
      if(!main.getPref("sender"))main.setPref("sender",main.userData[DATA_USER_MDN]);
      main.userData[DATA_USER_KEY]=t[7];
      this.reader.getHtml(this,"http://sms.nate.com/nateon30/nateonsms.jsp?TICKET="+t[7]+"&ID="+username+"&mobile=");
    }else main.showError(aData,aStatus,aHttp);
    break;
  case 4:
    this.reader.getHtml(this,"http://sms.nate.com/nateon2007/sms.jsp?t="+main.userData[DATA_USER_KEY]);
    break;
  case 5:
    var fnd=aData.match(/freeAmount\s*=\s*"(\S*)"/);
    if(fnd){
      main.userData[DATA_FREE_COUNT]=parseInt(fnd[1]);
      fnd=aData.match(/couponAmount\s*=\s*"(\S*)"/);
      if(fnd){
        main.userData[DATA_COUPON_COUNT]=parseInt(fnd[1]);
        main.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_SUCC);
        main.setState(nsILightSMS.ST_FREECOUNT,main.userData[DATA_FREE_COUNT]+main.userData[DATA_COUPON_COUNT]);
        main.stage=STAGE_READY;
        if(main.cmd==CMD_SEND){
          this.handler.send();
        }else main.onreq=false;
        break;
      }
    }
    main.showError(aData,aStatus,aHttp);
    break;
  case STAGE_SEND:
    fnd=aData.match(/resultWindow2.jsp\?display=MMS&ofr=(-*\d+)/);
    if(aStatus==200&&fnd){
      main.onSendSucc();
    }else{
      main.setState(nsILightSMS.ST_SEND,nsILightSMS.RS_FAIL);
    }
    main.onreq=false;
    break;
  }
  if(main.stage<STAGE_READY)main.stage++;
}
SMS_NateOn.prototype.send=function(){
  var main=this.main;
  main.stage=STAGE_SEND;
  var sender=main.getPref("sender");
  var receiver=main.getNumber(main.recv);
  var str=main.msg;
  var msg="";
  for(var i=0;i<str.length;i++){
    var n=str.charCodeAt(i);
    if(n<=0xff){
      msg+=str.charAt(i);
    }else{
      var t=n.toString(16).toUpperCase();
      for(var j=t.length;j<4;j++){
        t="0"+t;
      }
      msg+="%u"+t;
    }
  }
  var post="msg=&isFirst=first"
  var url="http://sms.nate.com/servlets/NateonSparrow?receivenum="
            +receiver+"&receivename=%20&v_message="
            +msg+"&sendnum="+sender+"&rsvdate=000000000000&display=MMS&freeAmount="
            +main.userData[DATA_FREE_COUNT]+"&notFree=false";
  main.setState(nsILightSMS.ST_SEND,nsILightSMS.RS_TRY);
  this.reader.getHtml(this,url,post);
}
SMS_NateOn.prototype._toHexString=function(charCode){
    return ("0" + charCode.toString(16)).slice(-2);
}
SMS_NateOn.prototype._MD5=function(str){
  var ch = Components.classes["@mozilla.org/security/hash;1"]
                   .createInstance(Components.interfaces.nsICryptoHash);
  var converter =
    Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
      createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
  converter.charset = "UTF-8";
  var result = {};
  var data = converter.convertToByteArray(str, result);
  ch.init(ch.MD5);
  ch.update(data,data.length);
  var hash=ch.finish(false);
  var s="";
  for(i in hash){
	  s+=this._toHexString(hash.charCodeAt(i));
  }
  return s;
}
